@@ -3,19 +3,20 @@
    This file is part of the GNU C Library.
    Written by Miles Bader <miles@gnu.ai.mit.edu>.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   The GNU C Library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2.1 of the License, or (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   The GNU C Library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU General Public License along
-   with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU Lesser General Public
+   License along with the GNU C Library; if not, write to the Free
+   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+   02111-1307 USA.  */
 
 #ifndef _GNU_SOURCE
 # define _GNU_SOURCE	1
@@ -68,13 +69,15 @@ char *alloca ();
 # endif
 #endif
 
-#ifndef __attribute
-# define __attribute(xyz)     /* Ignore */
-#endif
-
 #ifndef _LIBC
-# if !HAVE_DECL_STRERROR
-char *strerror ();
+# if HAVE_STRERROR_R
+#  if !HAVE_DECL_STRERROR_R
+char *strerror_r (int errnum, char *buf, size_t buflen);
+#  endif
+# else
+#  if !HAVE_DECL_STRERROR
+char *strerror (int errnum);
+#  endif
 # endif
 #endif
 
@@ -555,7 +558,7 @@ hol_entry_short_iterate (const struct hol_entry *entry,
 }
 
 static inline int
-__attribute ((always_inline))
+__attribute__ ((always_inline))
 hol_entry_long_iterate (const struct hol_entry *entry,
 			int (*func)(const struct argp_option *opt,
 				    const struct argp_option *real,
@@ -1538,7 +1541,9 @@ _help (const struct argp *argp, const struct argp_state *state, FILE *stream,
   if (! stream)
     return;
 
+#if _LIBC || (HAVE_FLOCKFILE && HAVE_FUNLOCKFILE)
   __flockfile (stream);
+#endif
 
   if (! uparams.valid)
     fill_in_uparams (state);
@@ -1546,7 +1551,9 @@ _help (const struct argp *argp, const struct argp_state *state, FILE *stream,
   fs = __argp_make_fmtstream (stream, 0, uparams.rmargin, 0);
   if (! fs)
     {
+#if _LIBC || (HAVE_FLOCKFILE && HAVE_FUNLOCKFILE)
       __funlockfile (stream);
+#endif
       return;
     }
 
@@ -1654,7 +1661,9 @@ Try `%s --help' or `%s --usage' for more information.\n"),
       anything = 1;
     }
 
+#if _LIBC || (HAVE_FLOCKFILE && HAVE_FUNLOCKFILE)
   __funlockfile (stream);
+#endif
 
   if (hol)
     hol_free (hol);
@@ -1662,9 +1671,6 @@ Try `%s --help' or `%s --usage' for more information.\n"),
   __argp_fmtstream_free (fs);
 }
 
-#ifndef weak_alias
-# define __argp_help argp_help
-#endif
 /* Output a usage message for ARGP to STREAM.  FLAGS are from the set
    ARGP_HELP_*.  NAME is what to use wherever a `program name' is needed. */
 void __argp_help (const struct argp *argp, FILE *stream,
@@ -1676,21 +1682,21 @@ void __argp_help (const struct argp *argp, FILE *stream,
 weak_alias (__argp_help, argp_help)
 #endif
 
-char *__argp_basename(char *name)
+#ifndef _LIBC
+char *__argp_basename (char *name)
 {
-  char *short_name = strrchr(name, '/');
+  char *short_name = strrchr (name, '/');
   return short_name ? short_name + 1 : name;
 }
+#endif
 
 char *
-__argp_short_program_name(const struct argp_state *state)
+__argp_short_program_name (void)
 {
-  if (state)
-    return state->name;
-#if HAVE_DECL_PROGRAM_INVOCATION_SHORT_NAME || defined _LIBC
+#if defined _LIBC || HAVE_DECL_PROGRAM_INVOCATION_SHORT_NAME
   return program_invocation_short_name;
 #elif HAVE_DECL_PROGRAM_INVOCATION_NAME
-  return __argp_basename(program_invocation_name);
+  return __argp_basename (program_invocation_name);
 #else
   /* FIXME: What now? Miles suggests that it is better to use NULL,
      but currently the value is passed on directly to fputs_unlocked,
@@ -1702,9 +1708,6 @@ __argp_short_program_name(const struct argp_state *state)
 #endif
 }
 
-#ifndef weak_alias
-# define __argp_state_help argp_state_help
-#endif
 /* Output, if appropriate, a usage message for STATE to STREAM.  FLAGS are
    from the set ARGP_HELP_*.  */
 void
@@ -1716,7 +1719,7 @@ __argp_state_help (const struct argp_state *state, FILE *stream, unsigned flags)
 	flags |= ARGP_HELP_LONG_ONLY;
 
       _help (state ? state->root_argp : 0, state, stream, flags,
-	     state ? state->name : __argp_short_program_name (state));
+	     state ? state->name : __argp_short_program_name ());
 
       if (!state || ! (state->flags & ARGP_NO_EXIT))
 	{
@@ -1731,9 +1734,6 @@ __argp_state_help (const struct argp_state *state, FILE *stream, unsigned flags)
 weak_alias (__argp_state_help, argp_state_help)
 #endif
 
-#ifndef weak_alias
-# define __argp_error argp_error
-#endif
 /* If appropriate, print the printf string FMT and following args, preceded
    by the program name and `:', to stderr, and followed by a `Try ... --help'
    message, then exit (1).  */
@@ -1748,7 +1748,9 @@ __argp_error (const struct argp_state *state, const char *fmt, ...)
 	{
 	  va_list ap;
 
+#if _LIBC || (HAVE_FLOCKFILE && HAVE_FUNLOCKFILE)
 	  __flockfile (stream);
+#endif
 
 	  va_start (ap, fmt);
 
@@ -1760,8 +1762,7 @@ __argp_error (const struct argp_state *state, const char *fmt, ...)
 	      __asprintf (&buf, fmt, ap);
 
 	      __fwprintf (stream, L"%s: %s\n",
-			  state ? state->name :
-			  __argp_short_program_name (state),
+			  state ? state->name : __argp_short_program_name (),
 			  buf);
 
 	      free (buf);
@@ -1770,8 +1771,7 @@ __argp_error (const struct argp_state *state, const char *fmt, ...)
 #endif
 	    {
 	      fputs_unlocked (state
-			      ? state->name :
-			      __argp_short_program_name (state),
+			      ? state->name : __argp_short_program_name (),
 			      stream);
 	      putc_unlocked (':', stream);
 	      putc_unlocked (' ', stream);
@@ -1785,7 +1785,9 @@ __argp_error (const struct argp_state *state, const char *fmt, ...)
 
 	  va_end (ap);
 
+#if _LIBC || (HAVE_FLOCKFILE && HAVE_FUNLOCKFILE)
 	  __funlockfile (stream);
+#endif
 	}
     }
 }
@@ -1793,9 +1795,6 @@ __argp_error (const struct argp_state *state, const char *fmt, ...)
 weak_alias (__argp_error, argp_error)
 #endif
 
-#ifndef weak_alias
-# define __argp_failure argp_failure
-#endif
 /* Similar to the standard gnu error-reporting function error(), but will
    respect the ARGP_NO_EXIT and ARGP_NO_ERRS flags in STATE, and will print
    to STATE->err_stream.  This is useful for argument parsing code that is
@@ -1814,17 +1813,18 @@ __argp_failure (const struct argp_state *state, int status, int errnum,
 
       if (stream)
 	{
+#if _LIBC || (HAVE_FLOCKFILE && HAVE_FUNLOCKFILE)
 	  __flockfile (stream);
+#endif
 
 #ifdef USE_IN_LIBIO
 	  if (_IO_fwide (stream, 0) > 0)
 	    __fwprintf (stream, L"%s",
-			state ? state->name :
-			__argp_short_program_name (state));
+			state ? state->name : __argp_short_program_name ());
 	  else
 #endif
 	    fputs_unlocked (state
-			    ? state->name : __argp_short_program_name (state),
+			    ? state->name : __argp_short_program_name (),
 			    stream);
 
 	  if (fmt)
@@ -1861,18 +1861,14 @@ __argp_failure (const struct argp_state *state, int status, int errnum,
 
 #ifdef USE_IN_LIBIO
 	      if (_IO_fwide (stream, 0) > 0)
-#if defined _LIBC
 		__fwprintf (stream, L": %s",
 			    __strerror_r (errnum, buf, sizeof (buf)));
-#else
-		__fwprintf (stream, L": %s", strerror (errnum));
-#endif
 	      else
 #endif
 		{
 		  putc_unlocked (':', stream);
 		  putc_unlocked (' ', stream);
-#if defined _LIBC
+#if defined _LIBC || defined HAVE_STRERROR_R
 		  fputs (__strerror_r (errnum, buf, sizeof (buf)), stream);
 #else
 		  fputs (strerror (errnum), stream);
@@ -1887,7 +1883,9 @@ __argp_failure (const struct argp_state *state, int status, int errnum,
 #endif
 	    putc_unlocked ('\n', stream);
 
+#if _LIBC || (HAVE_FLOCKFILE && HAVE_FUNLOCKFILE)
 	  __funlockfile (stream);
+#endif
 
 	  if (status && (!state || !(state->flags & ARGP_NO_EXIT)))
 	    exit (status);
