@@ -349,19 +349,21 @@ shishi_authenticator_client_set (Shishi * handle,
   return SHISHI_OK;
 }
 
+/**
+ * shishi_authenticator_ctime:
+ * @handle: shishi handle as allocated by shishi_init().
+ * @authenticator: Authenticator as allocated by shishi_authenticator().
+ * @ctime: newly allocated zero-terminated character array with client time.
+ *
+ * Extract client time from Authenticator.
+ *
+ * Return value: Returns SHISHI_OK iff successful.
+ **/
 int
-shishi_authenticator_ctime_get (Shishi * handle,
-				Shishi_asn1 authenticator, char *ctime)
+shishi_authenticator_ctime (Shishi * handle,
+			    Shishi_asn1 authenticator, char **ctime)
 {
-  int len;
-  int res;
-
-  len = GENERALIZEDTIME_TIME_LEN + 1;
-  res = shishi_asn1_read (handle, authenticator, "ctime", ctime, &len);
-  if (res == SHISHI_OK && len == GENERALIZEDTIME_TIME_LEN)
-    ctime[len] = '\0';
-
-  return res;
+  return shishi_time (handle, authenticator, "ctime", ctime);
 }
 
 /**
@@ -437,7 +439,7 @@ shishi_authenticator_cusec_set (Shishi * handle,
 int
 shishi_authenticator_cname_get (Shishi * handle,
 				Shishi_asn1 authenticator,
-				char *cname, int *cnamelen)
+				char *cname, size_t * cnamelen)
 {
   return shishi_principal_name_get (handle, authenticator,
 				    "cname", cname, cnamelen);
@@ -446,7 +448,7 @@ shishi_authenticator_cname_get (Shishi * handle,
 int
 shishi_authenticator_cnamerealm_get (Shishi * handle,
 				     Shishi_asn1 authenticator,
-				     char *cnamerealm, int *cnamerealmlen)
+				     char *cnamerealm, size_t * cnamerealmlen)
 {
   return shishi_principal_name_realm_get (handle, authenticator,
 					  "cname",
@@ -557,15 +559,42 @@ int
 shishi_authenticator_add_cksum (Shishi * handle,
 				Shishi_asn1 authenticator,
 				Shishi_key * key,
-				int keyusage, char *data, int datalen)
+				int keyusage, char *data, size_t datalen)
+{
+  return shishi_authenticator_add_cksum_type (handle, authenticator, key,
+					      keyusage,
+					      shishi_cipher_defaultcksumtype
+					      (shishi_key_type (key)),
+					      data, datalen);
+}
+
+/**
+ * shishi_authenticator_add_cksum_type:
+ * @handle: shishi handle as allocated by shishi_init().
+ * @authenticator: authenticator as allocated by shishi_authenticator().
+ * @key: key to to use for encryption.
+ * @keyusage: kerberos key usage value to use in encryption.
+ * @cksumtype: checksum to type to calculate checksum.
+ * @data: input array with data to calculate checksum on.
+ * @datalen: size of input array with data to calculate checksum on.
+ *
+ * Calculate checksum for data and store it in the authenticator.
+ *
+ * Return value: Returns SHISHI_OK iff successful.
+ **/
+int
+shishi_authenticator_add_cksum_type (Shishi * handle,
+				     Shishi_asn1 authenticator,
+				     Shishi_key * key,
+				     int keyusage, int cksumtype,
+				     char *data, size_t datalen)
 {
   int res;
 
   if (data && datalen > 0)
     {
       char *cksum;
-      int cksumlen;
-      int cksumtype = shishi_cipher_defaultcksumtype (shishi_key_type (key));
+      size_t cksumlen;
 
       res = shishi_checksum (handle, key, keyusage, cksumtype,
 			     data, datalen, &cksum, &cksumlen);
@@ -621,7 +650,7 @@ int
 shishi_authenticator_add_authorizationdata (Shishi * handle,
 					    Shishi_asn1 authenticator,
 					    int adtype,
-					    char *addata, int addatalen)
+					    char *addata, size_t addatalen)
 {
   char *format;
   int res;
@@ -673,7 +702,8 @@ int
 shishi_authenticator_authorizationdata (Shishi * handle,
 					Shishi_asn1 authenticator,
 					int *adtype,
-					char *addata, int *addatalen, int nth)
+					char *addata, size_t * addatalen,
+					int nth)
 {
   char *format;
   int res;
@@ -712,7 +742,8 @@ shishi_authenticator_authorizationdata (Shishi * handle,
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_authenticator_remove_subkey (Shishi * handle, Shishi_asn1 authenticator)
+shishi_authenticator_remove_subkey (Shishi * handle,
+				    Shishi_asn1 authenticator)
 {
   int res;
 
@@ -741,12 +772,11 @@ shishi_authenticator_get_subkey (Shishi * handle,
 {
   int res;
   int subkeytype;
-  char * subkeyvalue;
+  char *subkeyvalue;
   size_t subkeylen;
   int n;
 
-  res = shishi_asn1_number_of_elements (handle, authenticator,
-					"subkey", &n);
+  res = shishi_asn1_number_of_elements (handle, authenticator, "subkey", &n);
   if (res != SHISHI_OK)
     return res;
 
@@ -756,7 +786,7 @@ shishi_authenticator_get_subkey (Shishi * handle,
     return res;
 
   res = shishi_asn1_read2 (handle, authenticator, "subkey.keyvalue",
-			  &subkeyvalue, &subkeylen);
+			   &subkeyvalue, &subkeylen);
   if (res != SHISHI_OK)
     return res;
 
@@ -788,9 +818,9 @@ shishi_authenticator_get_subkey (Shishi * handle,
  **/
 int
 shishi_authenticator_set_subkey (Shishi * handle,
-				Shishi_asn1 authenticator,
-				int32_t subkeytype,
-				char *subkey, size_t subkeylen)
+				 Shishi_asn1 authenticator,
+				 int32_t subkeytype,
+				 char *subkey, size_t subkeylen)
 {
   int res;
 
@@ -821,12 +851,12 @@ shishi_authenticator_add_random_subkey (Shishi * handle,
 					Shishi_asn1 authenticator)
 {
   int res;
-  int * etypes;
-  Shishi_key * subkey;
+  int *etypes;
+  Shishi_key *subkey;
 
   res = shishi_cfg_clientkdcetype (handle, &etypes);
   if (!res)
-      return res;
+    return res;
 
   res = shishi_key_random (handle, etypes[0], &subkey);
   if (res != SHISHI_OK)
@@ -837,7 +867,7 @@ shishi_authenticator_add_random_subkey (Shishi * handle,
 					 shishi_key_value (subkey),
 					 shishi_key_length (subkey));
 
-  shishi_key_done (&subkey);
+  shishi_key_done (subkey);
 
   return res;
 }
@@ -858,11 +888,11 @@ shishi_authenticator_add_subkey (Shishi * handle,
 				 Shishi_key * subkey)
 {
   int res;
-  int * etypes;
+  int *etypes;
 
   res = shishi_cfg_clientkdcetype (handle, &etypes);
   if (res != SHISHI_OK)
-      return res;
+    return res;
 
   res = shishi_authenticator_set_subkey (handle, authenticator,
 					 shishi_key_type (subkey),
