@@ -27,7 +27,7 @@ enum
   DEFAULT_PRINCIPAL_OPTION,
   CLIENT_KDC_ETYPES_OPTION,
   REALM_KDC_OPTION,
-  KDC_OPTION,
+  SERVER_REALM_OPTION,
   KDC_TIMEOUT_OPTION,
   KDC_RETRIES_OPTION,
   TICKET_LIFE_OPTION,
@@ -45,7 +45,7 @@ static char *const _shishi_opts[] = {
   /* [DEFAULT_PRINCIPAL_OPTION] = */ "default-principal",
   /* [CLIENT_KDC_ETYPES_OPTION] = */ "client-kdc-etypes",
   /* [REALM_KDC_OPTION] =         */ "realm-kdc",
-  /* [KDC_OPTION] =               */ "kdc",
+  /* [SERVER_REALM_OPTION] =      */ "server-realm",
   /* [KDC_TIMEOUT_OPTION] =       */ "kdc-timeout",
   /* [KDC_RETRIES_OPTION] =       */ "kdc-retries",
   /* [TICKET_LIFE_OPTION] =       */ "ticket-life",
@@ -72,6 +72,26 @@ shishi_realminfo (Shishi * handle, const char *realm)
   return NULL;
 }
 
+struct Shishi_realminfo *
+shishi_realminfo_new (Shishi * handle, char *realm)
+{
+  struct Shishi_realminfo *ri;
+
+  ri = shishi_realminfo (handle, realm);
+  if (ri)
+    return ri;
+
+  handle->realminfos = xrealloc (handle->realminfos,
+				 (++handle->nrealminfos) *
+				 sizeof (*handle->realminfos));
+
+  ri = &handle->realminfos[handle->nrealminfos - 1];
+  memset (ri, 0, sizeof (*ri));
+  ri->name = realm;
+
+  return ri;
+}
+
 /**
  * shishi_cfg:
  * @handle: Shishi library handle create by shishi_init().
@@ -149,7 +169,7 @@ shishi_cfg (Shishi * handle, char *option)
 	  break;
 
 	case REALM_KDC_OPTION:
-	  realm = strdup (value);
+	  realm = xstrdup (value);
 	  for (i = 0; i < handle->nrealminfos; i++)
 	    if (strcmp (realm, handle->realminfos[i].name) == 0)
 	      {
@@ -163,23 +183,32 @@ shishi_cfg (Shishi * handle, char *option)
 		  }
 		break;
 	      }
-	  handle->realminfos = realloc (handle->realminfos,
-					(handle->nrealminfos + 1) *
-					sizeof (*handle->realminfos));
-	  if (handle->realminfos == NULL)
-	    return SHISHI_MALLOC_ERROR;
+	  handle->realminfos = xrealloc (handle->realminfos,
+					 (handle->nrealminfos + 1) *
+					 sizeof (*handle->realminfos));
 	  handle->realminfos[handle->nrealminfos].name = realm;
 	  handle->realminfos[handle->nrealminfos].kdcaddresses = NULL;
 	  handle->realminfos[handle->nrealminfos].nkdcaddresses = 0;
 	  handle->nrealminfos++;
 	  break;
 
+	case SERVER_REALM_OPTION:
+	  {
+	    struct Shishi_realminfo *ri;
+	    ri = shishi_realminfo_new (handle, value);
+	    ri->serverwildcards = xrealloc (ri->serverwildcards,
+					    ++ri->nserverwildcards *
+					    sizeof (*ri->serverwildcards));
+	    ri->serverwildcards[ri->nserverwildcards - 1] = xstrdup (value);
+	  }
+	  break;
+
 	case DEFAULT_REALM_OPTION:
-	  handle->default_realm = strdup (value);
+	  handle->default_realm = xstrdup (value);
 	  break;
 
 	case DEFAULT_PRINCIPAL_OPTION:
-	  handle->default_principal = strdup (value);
+	  handle->default_principal = xstrdup (value);
 	  break;
 
 	case CLIENT_KDC_ETYPES_OPTION:
@@ -188,12 +217,8 @@ shishi_cfg (Shishi * handle, char *option)
 	    return res;
 	  break;
 
-	case KDC_OPTION:
-	  handle->kdc = strdup (value);
-	  break;
-
 	case STRINGPROCESS_OPTION:
-	  handle->stringprocess = strdup (value);
+	  handle->stringprocess = xstrdup (value);
 	  break;
 
 	case VERBOSE_OPTION:
@@ -230,11 +255,11 @@ shishi_cfg (Shishi * handle, char *option)
 		  {
 		    *p = '\0';
 		    p++;
-		    if (strcmp(p, "udp") == 0)
+		    if (strcmp (p, "udp") == 0)
 		      protocol = UDP;
-		    else if (strcmp(p, "tcp") == 0)
+		    else if (strcmp (p, "tcp") == 0)
 		      protocol = TCP;
-		    else if (strcmp(p, "tls") == 0)
+		    else if (strcmp (p, "tls") == 0)
 		      protocol = TLS;
 		    else
 		      shishi_warn (handle,
@@ -254,7 +279,7 @@ shishi_cfg (Shishi * handle, char *option)
 		ri->kdcaddresses = xrealloc (ri->kdcaddresses,
 					     (ri->nkdcaddresses + 1) *
 					     sizeof (*ri->kdcaddresses));
-		ri->kdcaddresses[ri->nkdcaddresses].name = strdup (value);
+		ri->kdcaddresses[ri->nkdcaddresses].name = xstrdup (value);
 		ri->kdcaddresses[ri->nkdcaddresses].protocol = protocol;
 		sinaddr = (struct sockaddr_in *)
 		  &ri->kdcaddresses[ri->nkdcaddresses].sockaddress;
@@ -494,10 +519,8 @@ shishi_cfg_clientkdcetype_set (Shishi * handle, char *value)
 	  int *new;
 
 	  tot++;
-	  new = realloc (handle->clientkdcetypes,
-			 tot * sizeof (*handle->clientkdcetypes));
-	  if (handle->clientkdcetypes == NULL)
-	    return SHISHI_MALLOC_ERROR;
+	  new = xrealloc (handle->clientkdcetypes,
+			  tot * sizeof (*handle->clientkdcetypes));
 	  handle->clientkdcetypes = new;
 	  handle->clientkdcetypes[tot - 1] = etype;
 	  handle->nclientkdcetypes = tot;
