@@ -17,202 +17,41 @@
  * along with Shishi; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * Note: This file is #include'd by crypto.c.
- *
  */
 
-static int
-raw_des_verify (Shishi * handle, int algo, char *out, int *outlen)
-{
-#ifdef USE_GCRYPT
-  char md[MAX_HASH_LEN];
-  gcry_md_hd_t hd;
-  gpg_error_t err;
-  int alg = (algo == SHISHI_DES_CBC_MD4) ? GCRY_MD_MD4 : GCRY_MD_MD5;
-  int hlen = gcry_md_get_algo_dlen (alg);
-  int ok;
-  char *p;
+#include "internal.h"
 
-  memcpy (md, out + 8, hlen);
-  memset (out + 8, 0, hlen);
-
-  err = gcry_md_open (&hd, alg, 0);
-  if (err != GPG_ERR_NO_ERROR)
-    {
-      shishi_error_printf (handle, "Algo %d not available in libgcrypt", alg);
-      return SHISHI_CRYPTO_INTERNAL_ERROR;
-    }
-
-  gcry_md_write (hd, out, *outlen);
-
-  p = gcry_md_read (hd, alg);
-  if (p == NULL)
-    {
-      shishi_error_printf (handle, "Libgcrypt failed to compute hash");
-      return SHISHI_CRYPTO_INTERNAL_ERROR;
-    }
-
-  if (VERBOSECRYPTO (handle))
-    {
-      puts ("DES verify:");
-      hexprint (md, hlen);
-      puts ("");
-      hexprint (p, hlen);
-      puts ("");
-    }
-
-  ok = memcmp (p, md, hlen) == 0;
-
-  gcry_md_close (hd);
-
-  if (!ok)
-    {
-      shishi_error_printf (handle, "DES verify failed");
-      return SHISHI_CRYPTO_ERROR;
-    }
-
-  memmove (out, out + 8 + hlen, *outlen - 8 - hlen);
-  *outlen -= 8 + hlen;
-#else
-  struct md5_ctx md5;
-  struct md4_ctx md4;
-  char incoming[MAX_HASH_LEN];
-  char computed[MAX_HASH_LEN];
-  int hlen;
-
-  switch (algo)
-    {
-    case SHISHI_DES_CBC_MD4:
-      memcpy (incoming, out + 8, MD4_DIGEST_SIZE);
-      memset (out + 8, 0, MD4_DIGEST_SIZE);
-      md4_init (&md4);
-      md4_update (&md4, *outlen, out);
-      md4_digest (&md4, MD4_DIGEST_SIZE, computed);
-      hlen = MD4_DIGEST_SIZE;
-      break;
-
-    case SHISHI_DES_CBC_MD5:
-      memcpy (incoming, out + 8, MD5_DIGEST_SIZE);
-      memset (out + 8, 0, MD5_DIGEST_SIZE);
-      md5_init (&md5);
-      md5_update (&md5, *outlen, out);
-      md5_digest (&md5, MD5_DIGEST_SIZE, computed);
-      hlen = MD5_DIGEST_SIZE;
-      break;
-
-    default:
-      shishi_error_printf (handle, "MD %d unknown in raw des verify", algo);
-      return SHISHI_CRYPTO_INTERNAL_ERROR;
-      break;
-    }
-
-  if (VERBOSECRYPTO (handle))
-    {
-      puts ("DES verify:");
-      hexprint (incoming, hlen);
-      puts ("");
-      hexprint (computed, hlen);
-      puts ("");
-    }
-
-  if (memcmp (computed, incoming, hlen) != 0)
-    {
-      shishi_error_printf (handle, "DES hash verify failed");
-      return SHISHI_CRYPTO_ERROR;
-    }
-
-  memmove (out, out + 8 + hlen, *outlen - 8 - hlen);
-  *outlen -= 8 + hlen;
-#endif
-
-  return SHISHI_OK;
-}
+#include "crypto.h"
 
 static int
-raw_des_checksum (Shishi * handle,
-		  int algo,
-		  const char *in, size_t inlen,
-		  char *out, size_t * outlen, int hashzeros)
+raw_des_checksum0 (Shishi * handle, int algo,
+		   const char *in, size_t inlen, char *out, size_t * outlen)
 {
-#ifdef USE_GCRYPT
-  gpg_error_t err;
-  int alg = (algo == SHISHI_DES_CBC_MD4) ? GCRY_MD_MD4 : GCRY_MD_MD5;
-  int hlen = gcry_md_get_algo_dlen (alg);
-  char buffer[8 + MAX_HASH_LEN];
+  char *tmp;
+  size_t tmplen;
   char *p;
-  gcry_md_hd_t hd;
-  int res;
-
-  err = gcry_md_open (&hd, alg, 0);
-  if (err != GPG_ERR_NO_ERROR)
-    {
-      shishi_error_printf (handle, "MD %d not available in libgcrypt", alg);
-      return SHISHI_CRYPTO_INTERNAL_ERROR;
-    }
-
-  res = shishi_randomize (handle, buffer, 8);
-  if (res != SHISHI_OK)
-    return res;
-
-  memset (buffer + 8, 0, hlen);
-
-  if (hashzeros)
-    gcry_md_write (hd, buffer, 8 + hlen);
-  else
-    gcry_md_write (hd, buffer, 8);
-  gcry_md_write (hd, in, inlen);
-
-  p = gcry_md_read (hd, alg);
-  if (p == NULL)
-    {
-      shishi_error_printf (handle, "Libgcrypt failed to compute hash");
-      return SHISHI_CRYPTO_INTERNAL_ERROR;
-    }
-
-  memcpy (out, buffer, 8);
-  memcpy (out + 8, p, hlen);
-
-  gcry_md_close (hd);
-
-  *outlen = 8 + hlen;
-#else
-  struct md5_ctx md5;
-  struct md4_ctx md4;
-  int hlen;
+  int blen = 8;
+  int hlen = 16;
   int rc;
 
-  rc = shishi_randomize (handle, out, 8);
+  rc = shishi_randomize (handle, out, blen);
   if (rc != SHISHI_OK)
     return rc;
 
+  tmplen = blen + inlen;
+  tmp = xmalloc (tmplen);
+
+  memcpy (tmp, out, blen);
+  memcpy (tmp + blen, in, inlen);
+
   switch (algo)
     {
     case SHISHI_DES_CBC_MD4:
-      memset (out + 8, 0, MD4_DIGEST_SIZE);
-      md4_init (&md4);
-      if (hashzeros)
-	md4_update (&md4, 8 + MD4_DIGEST_SIZE, out);
-      else
-	md4_update (&md4, 8, out);
-      md4_update (&md4, inlen, in);
-
-      md4_digest (&md4, MD4_DIGEST_SIZE, out + 8);
-
-      hlen = MD4_DIGEST_SIZE;
+      rc = shishi_md4 (handle, tmp, tmplen, &p);
       break;
 
     case SHISHI_DES_CBC_MD5:
-      memset (out + 8, 0, MD5_DIGEST_SIZE);
-      md5_init (&md5);
-      if (hashzeros)
-	md5_update (&md5, 8 + MD5_DIGEST_SIZE, out);
-      else
-	md5_update (&md5, 8, out);
-      md5_update (&md5, inlen, in);
-
-      md5_digest (&md5, MD5_DIGEST_SIZE, out + 8);
-
-      hlen = MD5_DIGEST_SIZE;
+      rc = shishi_md5 (handle, tmp, tmplen, &p);
       break;
 
     default:
@@ -221,8 +60,55 @@ raw_des_checksum (Shishi * handle,
       break;
     }
 
-  *outlen = 8 + hlen;
-#endif
+  memcpy (out + blen, p, hlen);
+
+  *outlen = blen + hlen;
+
+  return SHISHI_OK;
+}
+
+static int
+raw_des_checksum1 (Shishi * handle, int algo,
+		   const char *in, size_t inlen, char *out, size_t * outlen)
+{
+  char *tmp;
+  size_t tmplen;
+  char *p;
+  int blen = 8;
+  int hlen = 16;
+  int rc;
+
+  rc = shishi_randomize (handle, out, blen);
+  if (rc != SHISHI_OK)
+    return rc;
+
+  memset (out + blen, 0, hlen);
+
+  tmplen = blen + hlen + inlen;
+  tmp = xmalloc (tmplen);
+
+  memcpy (tmp, out, blen + hlen);
+  memcpy (tmp + blen + hlen, in, inlen);
+
+  switch (algo)
+    {
+    case SHISHI_DES_CBC_MD4:
+      rc = shishi_md4 (handle, tmp, tmplen, &p);
+      break;
+
+    case SHISHI_DES_CBC_MD5:
+      rc = shishi_md5 (handle, tmp, tmplen, &p);
+      break;
+
+    default:
+      shishi_error_printf (handle, "MD %d unknown in raw des checksum", algo);
+      return SHISHI_CRYPTO_INTERNAL_ERROR;
+      break;
+    }
+
+  memcpy (out + blen, p, hlen);
+
+  *outlen = blen + hlen;
 
   return SHISHI_OK;
 }
@@ -250,7 +136,7 @@ des_encrypt_checksum (Shishi * handle,
   memcpy (inpad, in, inlen);
   memset (inpad + inlen, 0, padzerolen);
 
-  res = raw_des_checksum (handle, algo, inpad, inpadlen, cksum, &cksumlen, 1);
+  res = raw_des_checksum1 (handle, algo, inpad, inpadlen, cksum, &cksumlen);
   if (res != SHISHI_OK)
     {
       shishi_error_printf (handle, "DES checksum failed");
@@ -264,8 +150,9 @@ des_encrypt_checksum (Shishi * handle,
 
   free (inpad);
 
-  res = simplified_encrypt (handle, key, 0, iv, ivlen, ivout, ivoutlen,
-			    pt, ptlen, out, outlen);
+  res =
+    _shishi_simplified_encrypt (handle, key, 0, iv, ivlen, ivout, ivoutlen,
+				pt, ptlen, out, outlen);
 
   free (pt);
 
@@ -328,8 +215,8 @@ des_none_encrypt (Shishi * handle,
 		  char **ivout, size_t * ivoutlen,
 		  const char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return simplified_encrypt (handle, key, 0, iv, ivlen, ivout, ivoutlen,
-			     in, inlen, out, outlen);
+  return _shishi_simplified_encrypt (handle, key, 0, iv, ivlen, ivout,
+				     ivoutlen, in, inlen, out, outlen);
 }
 
 static int
@@ -342,22 +229,56 @@ des_decrypt_verify (Shishi * handle,
 		    char **out, size_t * outlen, int algo)
 {
   int res;
+  char incoming[16];
+  char *computed;
+  size_t hlen = 16;
 
-  res = simplified_decrypt (handle, key, 0, iv, ivlen, ivout, ivoutlen,
-			    in, inlen, out, outlen);
+  res =
+    _shishi_simplified_decrypt (handle, key, 0, iv, ivlen, ivout, ivoutlen,
+				in, inlen, out, outlen);
   if (res != SHISHI_OK)
     {
       shishi_error_printf (handle, "decrypt failed");
       return res;
     }
 
-  res = raw_des_verify (handle, algo, *out, outlen);
-  if (res != SHISHI_OK)
+  memcpy (incoming, *out + 8, hlen);
+  memset (*out + 8, 0, hlen);
+
+  switch (algo)
     {
-      shishi_error_printf (handle, "verify failed");
-      return res;
+    case SHISHI_DES_CBC_MD4:
+      shishi_md4 (handle, *out, *outlen, &computed);
+      break;
+
+    case SHISHI_DES_CBC_MD5:
+      shishi_md5 (handle, *out, *outlen, &computed);
+      break;
+
+    default:
+      shishi_error_printf (handle, "MD %d unknown in raw des verify", algo);
+      return SHISHI_CRYPTO_ERROR;
+      break;
     }
 
+  if (VERBOSECRYPTO (handle))
+    {
+      puts ("DES verify:");
+      _shishi_hexprint (incoming, hlen);
+      _shishi_hexprint (computed, hlen);
+    }
+
+  if (memcmp (computed, incoming, hlen) != 0)
+    {
+      shishi_error_printf (handle, "DES hash verify failed");
+      return SHISHI_CRYPTO_ERROR;
+    }
+
+  free (computed);
+
+  memmove (*out, *out + 8 + hlen, *outlen - 8 - hlen);
+  *outlen -= 8 + hlen;
+
   return SHISHI_OK;
 }
 
@@ -412,11 +333,11 @@ des_none_decrypt (Shishi * handle,
 		  char **ivout, size_t * ivoutlen,
 		  const char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return simplified_decrypt (handle, key, 0, iv, ivlen, ivout, ivoutlen,
-			     in, inlen, out, outlen);
+  return _shishi_simplified_decrypt (handle, key, 0, iv, ivlen, ivout,
+				     ivoutlen, in, inlen, out, outlen);
 }
 
-static int
+static void
 des_set_odd_key_parity (char key[8])
 {
   int i, j;
@@ -433,63 +354,47 @@ des_set_odd_key_parity (char key[8])
       if ((n_set_bits % 2) == 0)
 	key[i] |= 1;
     }
-
-  return SHISHI_OK;
 }
 
+static char weak_des_keys[16][8] = {
+  /* Weak keys */
+  "\x01\x01\x01\x01\x01\x01\x01\x01",
+  "\x1F\x1F\x1F\x1F\x0E\x0E\x0E\x0E",
+  "\xE0\xE0\xE0\xE0\xF1\xF1\xF1\xF1",
+  "\xFE\xFE\xFE\xFE\xFE\xFE\xFE\xFE",
+  /* Semiweak keys */
+  "\x01\xFE\x01\xFE\x01\xFE\x01\xFE",
+  "\x1F\xE0\x1F\xE0\x0E\xF1\x0E\xF1",
+  "\x01\xE0\x01\xE0\x01\xF1\x01\xF1",
+  "\x1F\xFE\x1F\xFE\x0E\xFE\x0E\xFE",
+  "\x01\x1F\x01\x1F\x01\x0E\x01\x0E",
+  "\xE0\xFE\xE0\xFE\xF1\xFE\xF1\xFE",
+  "\xFE\x01\xFE\x01\xFE\x01\xFE\x01",
+  "\xE0\x1F\xE1\x0F\xF1\x0E\xF1\x0E",
+  "\xE0\x01\xE0\x01\xF1\x01\xF1\x01",
+  "\xFE\x1F\xFE\x1F\xFE\x0E\xFE\x0E",
+  "\x1F\x01\x1F\x01\x0E\x01\x0E\x01",
+  "\xFE\xE0\xFE\xE0\xFE\xF1\xFE\xF1"
+};
+
 static int
-des_key_correction (Shishi * handle, char *key)
+des_key_correction (Shishi * handle, char key[8])
 {
-#ifdef USE_GCRYPT
-  gcry_cipher_hd_t ch;
-  gpg_error_t err;
+  size_t i;
 
   /* fixparity(key); */
   des_set_odd_key_parity (key);
 
-  err = gcry_cipher_open (&ch, GCRY_CIPHER_DES, GCRY_CIPHER_MODE_CBC, 0);
-  if (err != GPG_ERR_NO_ERROR)
-    {
-      shishi_error_printf (handle, "DES-CBC not available in libgcrypt");
-      return SHISHI_CRYPTO_INTERNAL_ERROR;
-    }
-
-  /* XXX? libgcrypt tests for pseudo-weak keys, rfc 1510 doesn't */
-
-  err = gcry_cipher_setkey (ch, key, 8);
-
-  gcry_cipher_close (ch);
-
-  if (err != GPG_ERR_NO_ERROR)
-    {
-      if (gpg_err_code (err) == GPG_ERR_WEAK_KEY)
-	{
-	  if (VERBOSECRYPTO (handle))
-	    printf ("\t ;; WEAK KEY (corrected)\n");
-	  key[7] ^= 0xF0;
-	}
-      else
-	{
-	  shishi_error_printf (handle, "DES setkey failed");
-	  shishi_error_set (handle, gpg_strerror (err));
-	  return SHISHI_CRYPTO_INTERNAL_ERROR;
-	}
-    }
-#else
-  struct CBC_MAC_CTX (struct des_ctx, DES_BLOCK_SIZE) des;
-  int rc;
-
-  /* fixparity(key); */
-  des_set_odd_key_parity (key);
-
-  rc = des_set_key (&des.ctx, key);
-  if (!rc && des.ctx.status == DES_WEAK_KEY)
-    {
-      if (VERBOSECRYPTO (handle))
-	printf ("\t ;; WEAK KEY (corrected)\n");
-      key[7] ^= 0xF0;
-    }
-#endif
+  /* This loop could be replaced by optimized code (compare nettle),
+     but let's not do that. */
+  for (i = 0; i < 16; i++)
+    if (memcmp (key, weak_des_keys[i], 8) == 0)
+      {
+	if (VERBOSECRYPTO (handle))
+	  printf ("\t ;; WEAK KEY (corrected)\n");
+	key[7] ^= 0xF0;
+	break;
+      }
 
   return SHISHI_OK;
 }
@@ -515,70 +420,6 @@ des_random_to_key (Shishi * handle,
   return SHISHI_OK;
 }
 
-static int
-des_cbc_check (Shishi * handle, char key[8], char *data, int n_data)
-{
-#ifdef USE_GCRYPT
-  gcry_cipher_hd_t ch;
-  gpg_error_t err;
-  int res = SHISHI_CRYPTO_INTERNAL_ERROR;
-
-  err = gcry_cipher_open (&ch, GCRY_CIPHER_DES,
-			  GCRY_CIPHER_MODE_CBC, GCRY_CIPHER_CBC_MAC);
-  if (err != GPG_ERR_NO_ERROR)
-    {
-      shishi_error_printf (handle, "DES-CBC-MAC not available in libgcrypt");
-      return SHISHI_CRYPTO_INTERNAL_ERROR;
-    }
-
-  err = gcry_cipher_setkey (ch, key, 8);
-  if (err != GPG_ERR_NO_ERROR)
-    {
-      shishi_error_printf (handle, "DES setkey failed");
-      shishi_error_set (handle, gpg_strerror (err));
-      goto done;
-    }
-
-  err = gcry_cipher_setiv (ch, key, 8);
-  if (err != GPG_ERR_NO_ERROR)
-    {
-      shishi_error_printf (handle, "DES setiv failed");
-      shishi_error_set (handle, gpg_strerror (err));
-      goto done;
-    }
-
-  err = gcry_cipher_encrypt (ch, key, 8, data, n_data);
-  if (err != GPG_ERR_NO_ERROR)
-    {
-      shishi_error_printf (handle, "DES encrypt failed");
-      shishi_error_set (handle, gpg_strerror (err));
-      goto done;
-    }
-
-  return SHISHI_OK;
-
-done:
-  gcry_cipher_close (ch);
-  return res;
-#else
-  struct CBC_MAC_CTX (struct des_ctx, DES_BLOCK_SIZE) des;
-  int rc;
-
-  rc = des_set_key (&des.ctx, key);
-  if (!rc)
-    {
-      shishi_error_printf (handle, "des_set_key() failed (%d)", rc);
-      return SHISHI_CRYPTO_INTERNAL_ERROR;
-    }
-
-  CBC_SET_IV (&des, key);
-
-  CBC_MAC (&des, des_encrypt, n_data, key, data);
-
-  return SHISHI_OK;
-#endif
-}
-
 static int
 des_string_to_key (Shishi * handle,
 		   const char *string,
@@ -590,6 +431,7 @@ des_string_to_key (Shishi * handle,
   int n_s;
   int odd;
   char tempkey[8];
+  char *p;
   int i, j;
   char temp, temp2;
   int res;
@@ -597,18 +439,12 @@ des_string_to_key (Shishi * handle,
   if (VERBOSECRYPTO (handle))
     {
       printf ("des_string_to_key (string, salt)\n");
-
       printf ("\t ;; String:\n");
-      escapeprint (string, stringlen);
-      hexprint (string, stringlen);
-      puts ("");
-      puts ("");
-
+      _shishi_escapeprint (string, stringlen);
+      _shishi_hexprint (string, stringlen);
       printf ("\t ;; Salt:\n");
-      escapeprint (salt, saltlen);
-      hexprint (salt, saltlen);
-      puts ("");
-
+      _shishi_escapeprint (salt, saltlen);
+      _shishi_hexprint (salt, saltlen);
       printf ("odd = 1;\n");
       printf ("s = string | salt;\n");
       printf ("tempstring = 0; /* 56-bit string */\n");
@@ -630,9 +466,8 @@ des_string_to_key (Shishi * handle,
   if (VERBOSECRYPTO (handle))
     {
       printf ("\t ;; s = pad(string|salt):\n");
-      escapeprint (s, n_s);
-      hexprint (s, n_s);
-      puts ("");
+      _shishi_escapeprint (s, n_s);
+      _shishi_hexprint (s, n_s);
     }
 
   for (i = 0; i < n_s / 8; i++)
@@ -642,11 +477,9 @@ des_string_to_key (Shishi * handle,
 	  printf ("for (8byteblock in s) {\n");
 	  printf ("\t ;; loop iteration %d\n", i);
 	  printf ("\t ;; 8byteblock:\n");
-	  escapeprint (&s[i * 8], 8);
-	  hexprint (&s[i * 8], 8);
-	  puts ("");
-	  binprint (&s[i * 8], 8);
-	  puts ("");
+	  _shishi_escapeprint (&s[i * 8], 8);
+	  _shishi_hexprint (&s[i * 8], 8);
+	  _shishi_binprint (&s[i * 8], 8);
 	  printf ("56bitstring = removeMSBits(8byteblock);\n");
 	}
 
@@ -656,8 +489,7 @@ des_string_to_key (Shishi * handle,
       if (VERBOSECRYPTO (handle))
 	{
 	  printf ("\t ;; 56bitstring:\n");
-	  bin7print (&s[i * 8], 8);
-	  puts ("");
+	  _shishi_bin7print (&s[i * 8], 8);
 	  printf ("if (odd == 0) reverse(56bitstring);\t ;; odd=%d\n", odd);
 	}
 
@@ -688,8 +520,7 @@ des_string_to_key (Shishi * handle,
 	    {
 	      printf ("reverse(56bitstring)\n");
 	      printf ("\t ;; 56bitstring after reverse\n");
-	      bin7print (&s[i * 8], 8);
-	      puts ("");
+	      _shishi_bin7print (&s[i * 8], 8);
 	    }
 	}
 
@@ -708,9 +539,7 @@ des_string_to_key (Shishi * handle,
       if (VERBOSECRYPTO (handle))
 	{
 	  printf ("\t ;; tempstring\n");
-	  bin7print (tempkey, 8);
-	  puts ("");
-	  puts ("");
+	  _shishi_bin7print (tempkey, 8);
 	}
     }
 
@@ -723,10 +552,8 @@ des_string_to_key (Shishi * handle,
       printf ("}\n");
       printf ("\t ;; for loop terminated\n");
       printf ("\t ;; tempstring as 64bitblock\n");
-      hexprint (tempkey, 8);
-      puts ("");
-      binprint (tempkey, 8);
-      puts ("");
+      _shishi_hexprint (tempkey, 8);
+      _shishi_binprint (tempkey, 8);
       printf ("/* add parity as low bit of each byte */\n");
       printf ("tempkey = key_correction(add_parity_bits(tempstring));\n");
     }
@@ -738,12 +565,9 @@ des_string_to_key (Shishi * handle,
   if (VERBOSECRYPTO (handle))
     {
       printf ("\t ;; tempkey\n");
-      escapeprint (tempkey, 8);
-      hexprint (tempkey, 8);
-      puts ("");
-      binprint (tempkey, 8);
-      puts ("");
-      puts ("");
+      _shishi_escapeprint (tempkey, 8);
+      _shishi_hexprint (tempkey, 8);
+      _shishi_binprint (tempkey, 8);
       printf ("key = key_correction(DES-CBC-check(s,tempkey));\n");
     }
 
@@ -752,9 +576,11 @@ des_string_to_key (Shishi * handle,
     memcpy (s + stringlen, salt, saltlen);
   memset (s + stringlen + saltlen, 0, n_s - stringlen - saltlen);
 
-  res = des_cbc_check (handle, tempkey, s, n_s);
+  res = shishi_des_cbc_mac (handle, tempkey, tempkey, s, n_s, &p);
   if (res != SHISHI_OK)
     return res;
+  memcpy (tempkey, p, 8);
+  free (p);
 
   res = des_key_correction (handle, tempkey);
   if (res != SHISHI_OK)
@@ -763,12 +589,9 @@ des_string_to_key (Shishi * handle,
   if (VERBOSECRYPTO (handle))
     {
       printf ("\t ;; key\n");
-      escapeprint (tempkey, 8);
-      hexprint (tempkey, 8);
-      puts ("");
-      binprint (tempkey, 8);
-      puts ("");
-      puts ("");
+      _shishi_escapeprint (tempkey, 8);
+      _shishi_hexprint (tempkey, 8);
+      _shishi_binprint (tempkey, 8);
     }
 
   shishi_key_value_set (outkey, tempkey);
@@ -790,7 +613,7 @@ des_checksum (Shishi * handle,
   int i;
   int res;
 
-  res = raw_des_checksum (handle, algo, in, inlen, cksum, &cksumlen, 0);
+  res = raw_des_checksum0 (handle, algo, in, inlen, cksum, &cksumlen);
   if (res != SHISHI_OK)
     {
       shishi_error_set (handle, "raw des checksum failed");
@@ -802,8 +625,8 @@ des_checksum (Shishi * handle,
   for (i = 0; i < 8; i++)
     keyp[i] ^= 0xF0;
 
-  res = simplified_dencrypt (handle, key, NULL, 0, NULL, NULL,
-			     cksum, cksumlen, out, outlen, 0);
+  res = _shishi_simplified_dencrypt (handle, key, NULL, 0, NULL, NULL,
+				     cksum, cksumlen, out, outlen, 0);
 
   for (i = 0; i < 8; i++)
     keyp[i] ^= 0xF0;
@@ -846,89 +669,39 @@ gss_des_checksum (Shishi * handle,
 		  int cksumtype,
 		  const char *in, size_t inlen, char **out, size_t * outlen)
 {
-#ifdef USE_GCRYPT
-  char *keyp;
   char *p;
-  int i;
-  gcry_md_hd_t hd;
-  gcry_cipher_hd_t ch;
-  int res;
-
-  gcry_md_open (&hd, GCRY_MD_MD5, 0);
-  if (!hd)
-    return SHISHI_CRYPTO_INTERNAL_ERROR;
-
-  gcry_md_write (hd, in, inlen);
-  p = gcry_md_read (hd, GCRY_MD_MD5);
-
-  keyp = shishi_key_value (key);
-
-  gcry_cipher_open (&ch, GCRY_CIPHER_DES,
-		    GCRY_CIPHER_MODE_CBC, GCRY_CIPHER_CBC_MAC);
-  if (ch == NULL)
-    return SHISHI_CRYPTO_INTERNAL_ERROR;
-
-  res = gcry_cipher_setkey (ch, keyp, 8);
-  if (res != GPG_ERR_NO_ERROR)
-    return SHISHI_CRYPTO_INTERNAL_ERROR;
-
-  res = gcry_cipher_setiv (ch, NULL, 8);
-  if (res != 0)
-    return SHISHI_CRYPTO_INTERNAL_ERROR;
-
-  *outlen = 8;
-  *out = xmalloc (*outlen);
-
-  res = gcry_cipher_encrypt (ch, *out, *outlen, p, 16);
-  if (res != 0)
-    return SHISHI_CRYPTO_INTERNAL_ERROR;
-
-  gcry_cipher_close (ch);
-  gcry_md_close (hd);
-#else
-  struct md5_ctx md5;
-  struct CBC_MAC_CTX (struct des_ctx, DES_BLOCK_SIZE) des;
-  char digest[MD5_DIGEST_SIZE];
   int rc;
 
-  md5_init (&md5);
-  md5_update (&md5, inlen, in);
-  md5_digest (&md5, sizeof (digest), digest);
+  rc = shishi_md5 (handle, in, inlen, &p);
+  if (rc != SHISHI_OK)
+    return rc;
 
-  rc = des_set_key (&des.ctx, shishi_key_value (key));
-  if (!rc)
-    {
-      shishi_error_printf (handle, "des_set_key() failed (%d)", rc);
-      return SHISHI_CRYPTO_INTERNAL_ERROR;
-    }
+  *outlen = 8;
+  rc = shishi_des_cbc_mac (handle, shishi_key_value (key), NULL, p, 16, out);
 
-  memset (des.iv, 0, sizeof (des.iv));
+  free (p);
 
-  *outlen = DES_BLOCK_SIZE;
-  *out = xmalloc (*outlen);
+  if (rc != SHISHI_OK)
+    return rc;
 
-  CBC_MAC (&des, des_encrypt, MD5_DIGEST_SIZE, *out, digest);
-#endif
   return SHISHI_OK;
 }
 
 static int
-des_md5_verify (Shishi * handle,
-		Shishi_key * key,
-		int keyusage,
-		int cksumtype,
-		const char *in, size_t inlen,
-		const char *cksum, size_t cksumlen)
+des_verify (Shishi * handle, int algo,
+	    Shishi_key * key,
+	    const char *in, size_t inlen, const char *cksum, size_t cksumlen)
 {
-  struct md5_ctx md5;
   char *out;
   size_t outlen;
-  char md[MD5_DIGEST_SIZE];
+  char *md;
+  size_t tmplen;
+  char *tmp;
   char *keyp;
   size_t i;
   int res;
 
-  if (cksumlen != 8 + MD5_DIGEST_SIZE)
+  if (cksumlen != 8 + 16)
     return SHISHI_VERIFY_FAILED;
 
   /*
@@ -942,8 +715,8 @@ des_md5_verify (Shishi * handle,
   for (i = 0; i < 8; i++)
     keyp[i] ^= 0xF0;
 
-  res = simplified_decrypt (handle, key, 0, NULL, 0, NULL, NULL,
-			    cksum, cksumlen, &out, &outlen);
+  res = _shishi_simplified_decrypt (handle, key, 0, NULL, 0, NULL, NULL,
+				    cksum, cksumlen, &out, &outlen);
 
   for (i = 0; i < 8; i++)
     keyp[i] ^= 0xF0;
@@ -954,14 +727,140 @@ des_md5_verify (Shishi * handle,
       return res;
     }
 
-  md5_init (&md5);
-  md5_update (&md5, 8, out);
-  md5_update (&md5, inlen, in);
+  tmplen = 8 + inlen;
+  tmp = xmalloc (tmplen);
+  memcpy (tmp, out, 8);
+  memcpy (tmp + 8, in, inlen);
 
-  md5_digest (&md5, MD5_DIGEST_SIZE, md);
+  switch (algo)
+    {
+    case SHISHI_RSA_MD4_DES:
+      res = shishi_md4 (handle, tmp, tmplen, &md);
+      break;
 
-  if (memcmp (out + 8, md, MD5_DIGEST_SIZE) != 0)
+    case SHISHI_RSA_MD5_DES:
+      res = shishi_md5 (handle, tmp, tmplen, &md);
+      break;
+
+    default:
+      res = SHISHI_CRYPTO_ERROR;
+    }
+
+  if (res != SHISHI_OK)
+    {
+      shishi_error_printf (handle, "DES verify MD error");
+      return res;
+    }
+
+  if (memcmp (out + 8, md, 16) != 0)
     return SHISHI_VERIFY_FAILED;
 
   return SHISHI_OK;
 }
+
+static int
+des_md4_verify (Shishi * handle,
+		Shishi_key * key,
+		int keyusage,
+		int cksumtype,
+		const char *in, size_t inlen,
+		const char *cksum, size_t cksumlen)
+{
+  return des_verify (handle, SHISHI_RSA_MD4_DES, key,
+		     in, inlen, cksum, cksumlen);
+}
+
+static int
+des_md5_verify (Shishi * handle,
+		Shishi_key * key,
+		int keyusage,
+		int cksumtype,
+		const char *in, size_t inlen,
+		const char *cksum, size_t cksumlen)
+{
+  return des_verify (handle, SHISHI_RSA_MD5_DES, key,
+		     in, inlen, cksum, cksumlen);
+}
+
+cipherinfo des_cbc_crc_info = {
+  SHISHI_DES_CBC_CRC,
+  "des-cbc-crc",
+  8,
+  4,
+  8,
+  8,
+  8,
+  SHISHI_RSA_MD5_DES,
+  des_random_to_key,
+  des_string_to_key,
+  des_crc_encrypt,
+  des_crc_decrypt
+};
+
+cipherinfo des_cbc_md4_info = {
+  SHISHI_DES_CBC_MD4,
+  "des-cbc-md4",
+  8,
+  0,
+  8,
+  8,
+  8,
+  SHISHI_RSA_MD4_DES,
+  des_random_to_key,
+  des_string_to_key,
+  des_md4_encrypt,
+  des_md4_decrypt
+};
+
+cipherinfo des_cbc_md5_info = {
+  SHISHI_DES_CBC_MD5,
+  "des-cbc-md5",
+  8,
+  0,
+  8,
+  8,
+  8,
+  SHISHI_RSA_MD5_DES,
+  des_random_to_key,
+  des_string_to_key,
+  des_md5_encrypt,
+  des_md5_decrypt
+};
+
+cipherinfo des_cbc_none_info = {
+  SHISHI_DES_CBC_NONE,
+  "des-cbc-none",
+  8,
+  0,
+  8,
+  3 * 8,
+  3 * 8,
+  SHISHI_RSA_MD5_DES,
+  des_random_to_key,
+  des_string_to_key,
+  des_none_encrypt,
+  des_none_decrypt
+};
+
+checksuminfo md4_des_info = {
+  SHISHI_RSA_MD4_DES,
+  "rsa-md4-des",
+  24,
+  des_md4_checksum,
+  des_md4_verify
+};
+
+checksuminfo md5_des_info = {
+  SHISHI_RSA_MD5_DES,
+  "rsa-md5-des",
+  24,
+  des_md5_checksum,
+  des_md5_verify
+};
+
+checksuminfo md5_gss_info = {
+  SHISHI_RSA_MD5_DES_GSS,
+  "rsa-md5-des-gss",
+  8,
+  gss_des_checksum
+};
