@@ -21,6 +21,16 @@
 
 #include "internal.h"
 
+/**
+ * shishi_encapreppart:
+ * @handle: shishi handle as allocated by shishi_init().
+ *
+ * This function creates a new EncAPRepPart, populated with some
+ * default values.  It uses the current time as returned by the system
+ * for the ctime and cusec fields.
+ *
+ * Return value: Returns the encapreppart or NULL on failure.
+ **/
 Shishi_asn1
 shishi_encapreppart (Shishi * handle)
 {
@@ -259,19 +269,21 @@ shishi_encapreppart_get_key (Shishi * handle,
   return SHISHI_OK;
 }
 
+/**
+ * shishi_encapreppart_ctime:
+ * @handle: shishi handle as allocated by shishi_init().
+ * @encapreppart: EncAPRepPart as allocated by shishi_encapreppart().
+ * @ctime: newly allocated zero-terminated character array with client time.
+ *
+ * Extract client time from EncAPRepPart.
+ *
+ * Return value: Returns SHISHI_OK iff successful.
+ **/
 int
-shishi_encapreppart_ctime_get (Shishi * handle,
-			       Shishi_asn1 encapreppart, char *ctime)
+shishi_encapreppart_ctime (Shishi * handle,
+			   Shishi_asn1 encapreppart, char **ctime)
 {
-  int len;
-  int res;
-
-  len = GENERALIZEDTIME_TIME_LEN + 1;
-  res = shishi_asn1_read (handle, encapreppart, "ctime", ctime, &len);
-  if (res == SHISHI_OK && len == GENERALIZEDTIME_TIME_LEN)
-    ctime[len] = '\0';
-
-  return res;
+  return shishi_time (handle, encapreppart, "ctime", ctime);
 }
 
 /**
@@ -369,6 +381,16 @@ shishi_encapreppart_seqnumber_get (Shishi * handle,
   return res;
 }
 
+/**
+ * shishi_encapreppart_time_copy:
+ * @handle: shishi handle as allocated by shishi_init().
+ * @encapreppart: EncAPRepPart as allocated by shishi_encapreppart().
+ * @authenticator: Authenticator to copy time fields from.
+ *
+ * Copy time fields from Authenticator into EncAPRepPart.
+ *
+ * Return value: Returns SHISHI_OK iff successful.
+ **/
 int
 shishi_encapreppart_time_copy (Shishi * handle,
 			       Shishi_asn1 encapreppart,
