@@ -119,6 +119,10 @@
 #include <resolv.h>
 #endif
 
+#ifdef HAVE_SYSLOG_H 
+#include <syslog.h> 
+#endif 
+
 #include "gettext.h"
 
 #include "base64.h"
@@ -188,6 +192,7 @@ struct Shishi
 {
   Shishi_asn1 asn1;
   int verbose;
+  int outputtype;
   char *default_realm;
   char *default_principal;
   size_t kdctimeout;
@@ -216,6 +221,9 @@ struct Shishi
 #define TICKETLIFE (60*60*8)	/* Work day */
 #define RENEWLIFE (60*60*24*7)	/* Week */
 
+extern int _shishi_crypto_init (Shishi * handle);
+extern int _shishi_asn1_init (Shishi * handle);
+
 int
 _shishi_print_armored_data (Shishi * handle,
 			    FILE * fh,
@@ -254,6 +262,11 @@ shishi_asn1_integer2_field (Shishi * handle,
 			    Shishi_asn1 node, unsigned long *i,
 			    const char *field);
 
+/* utils.c */
+extern void _shishi_escapeprint (const char *str, int len);
+extern void _shishi_hexprint (const char *str, int len);
+extern void _shishi_binprint (const char *str, int len);
+extern void _shishi_bin7print (const char *str, int len);
 extern time_t xtime (time_t * t);
 extern int xgettimeofday (struct timeval *tv, struct timezone *tz);
 
