@@ -47,11 +47,8 @@ shishi_safe (Shishi * handle, Shishi_safe ** safe)
   char *usec;
   int rc;
 
-  *safe = malloc (sizeof (**safe));
-  if (*safe == NULL)
-    return SHISHI_MALLOC_ERROR;
+  *safe = xcalloc (1, sizeof (**safe));
   lsafe = *safe;
-  memset (lsafe, 0, sizeof (*lsafe));
 
   lsafe->handle = handle;
   rc = shishi_key (handle, &lsafe->key);
@@ -103,6 +100,22 @@ shishi_safe (Shishi * handle, Shishi_safe ** safe)
   return SHISHI_OK;
 }
 
+/**
+ * shishi_safe_done:
+ * @safe: structure that holds information about SAFE exchange
+ *
+ * Deallocate resources associated with SAFE exchange.  This should be
+ * called by the application when it no longer need to utilize the
+ * SAFE exchange handle.
+ **/
+void
+shishi_safe_done (Shishi_safe * safe)
+{
+  shishi_asn1_done (safe->handle, safe->safe);
+  shishi_key_done (safe->key);
+  free (safe);
+}
+
 /**
  * shishi_safe_key:
  * @safe: structure that holds information about SAFE exchange
@@ -170,7 +183,7 @@ shishi_safe_safe_set (Shishi_safe * safe, Shishi_asn1 asn1safe)
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_safe_safe_der (Shishi_safe * safe, char **out, size_t *outlen)
+shishi_safe_safe_der (Shishi_safe * safe, char **out, size_t * outlen)
 {
   int rc;
 
@@ -334,7 +347,7 @@ shishi_safe_read (Shishi * handle, FILE * fh, Shishi_asn1 * safe)
  **/
 int
 shishi_safe_from_file (Shishi * handle, Shishi_asn1 * safe,
-		       int filetype, char *filename)
+		       int filetype, const char *filename)
 {
   int res;
   FILE *fh;
@@ -417,7 +430,7 @@ shishi_safe_cksum (Shishi * handle,
 int
 shishi_safe_set_cksum (Shishi * handle,
 		       Shishi_asn1 safe,
-		       int32_t cksumtype, char *cksum, size_t cksumlen)
+		       int32_t cksumtype, const char *cksum, size_t cksumlen)
 {
   int res;
 
@@ -446,12 +459,13 @@ shishi_safe_set_cksum (Shishi * handle,
  **/
 int
 shishi_safe_user_data (Shishi * handle,
-		       Shishi_asn1 safe, char **userdata, size_t * userdatalen)
+		       Shishi_asn1 safe, char **userdata,
+		       size_t * userdatalen)
 {
   int res;
 
   res = shishi_asn1_read2 (handle, safe, "safe-body.user-data",
-			  userdata, userdatalen);
+			   userdata, userdatalen);
   if (res != SHISHI_OK)
     return res;
 
@@ -472,7 +486,7 @@ shishi_safe_user_data (Shishi * handle,
 int
 shishi_safe_set_user_data (Shishi * handle,
 			   Shishi_asn1 safe,
-			   char *userdata, size_t userdatalen)
+			   const char *userdata, size_t userdatalen)
 {
   int res;
 
@@ -542,53 +556,43 @@ shishi_safe_build (Shishi_safe * safe, Shishi_key * key)
  * implementations use the RFC1510 algorithm.
  *
  * Return value: Returns SHISHI_OK iff successful,
- *               SHISHI_SAFE_BAD_KEYTYPE if an incompatible key type
- *               is used, or SHISHI_SAFE_VERIFY_FAILED if the actual
- *               verification failed.
+ *   SHISHI_SAFE_BAD_KEYTYPE if an incompatible key type is used, or
+ *   SHISHI_SAFE_VERIFY_FAILED if the actual verification failed.
  **/
 int
 shishi_safe_verify (Shishi_safe * safe, Shishi_key * key)
 {
-  char *cksum;
+  char *cksum = NULL;
   size_t cksumlen;
   int cksumtype;
-  char *cksum2;
-  size_t cksum2len;
-  int cksumtype2;
+  char *safeder = NULL;
+  size_t safederlen;
   int rc;
 
   rc = shishi_safe_cksum (safe->handle, safe->safe,
 			  &cksumtype, &cksum, &cksumlen);
   if (rc != SHISHI_OK)
-    goto error;
+    goto done;
 
-  rc = shishi_safe_build (safe, key);
+  rc = shishi_safe_set_cksum (safe->handle, safe->safe, 0, "", 0);
   if (rc != SHISHI_OK)
-    goto error;
+    goto done;
 
-  rc = shishi_safe_cksum (safe->handle, safe->safe,
-			  &cksumtype2, &cksum2, &cksum2len);
+  rc = shishi_safe_safe_der (safe, &safeder, &safederlen);
   if (rc != SHISHI_OK)
-    goto error;
+    goto done;
 
-  if (cksumtype != cksumtype2)
-    {
-      rc = SHISHI_SAFE_BAD_KEYTYPE;
-      goto error;
-    }
+  rc = shishi_verify (safe->handle, key, SHISHI_KEYUSAGE_KRB_SAFE,
+		      cksumtype, safeder, safederlen, cksum, cksumlen);
+  if (rc != SHISHI_OK)
+    goto done;
 
-  if (cksum2len != cksumlen || memcmp (cksum, cksum2, cksumlen) != 0)
-    {
-      rc = SHISHI_SAFE_VERIFY_FAILED;
-      goto error;
-    }
+  rc = SHISHI_OK;
 
-  return SHISHI_OK;
-
- error:
+done:
   if (cksum)
     free (cksum);
-  if (cksum2)
-    free (cksum2);
+  if (safeder)
+    free (safeder);
   return rc;
 }
