@@ -27,7 +27,7 @@
 #include <time.h>		/* time_t */
 #include <shishi-int.h>		/* uint32_t */
 
-#define SHISHI_VERSION "0.0.5"
+#define SHISHI_VERSION "0.0.6"
 
 /* Error codes */
 typedef enum
@@ -81,7 +81,8 @@ typedef enum
   SHISHI_CONNECT_ERROR = 46,
   SHISHI_VERIFY_FAILED = 47,
   SHISHI_PRIV_BAD_KEYTYPE = 48,
-  SHISHI_LAST_ERROR = 48
+  SHISHI_FILE_ERROR = 49,
+  SHISHI_LAST_ERROR = 49
 }
 Shishi_rc;
 
@@ -264,8 +265,8 @@ typedef enum
   SHISHI_DES3_CBC_HMAC_SHA1_KD = 16,
   SHISHI_AES128_CTS_HMAC_SHA1_96 = 17,
   SHISHI_AES256_CTS_HMAC_SHA1_96 = 18,
-  SHISHI_RC4_HMAC = 23,
-  SHISHI_RC4_HMAC_EXP = 24
+  SHISHI_ARCFOUR_HMAC = 23,
+  SHISHI_ARCFOUR_HMAC_EXP = 24
 }
 Shishi_etype;
 
@@ -283,7 +284,7 @@ typedef enum
   SHISHI_HMAC_SHA1_DES3_KD = 12,
   SHISHI_HMAC_SHA1_96_AES128 = 15,
   SHISHI_HMAC_SHA1_96_AES256 = 16,
-  SHISHI_RC4_HMAC_MD5 = -138,
+  SHISHI_ARCFOUR_HMAC_MD5 = -138,
   SHISHI_NO_CKSUMTYPE = -1
 }
 Shishi_cksumtype;
@@ -298,6 +299,14 @@ typedef enum
 }
 Shishi_filetype;
 
+typedef enum
+{
+  SHISHI_OUTPUTTYPE_NULL = 0,
+  SHISHI_OUTPUTTYPE_STDERR,
+  SHISHI_OUTPUTTYPE_SYSLOG
+}
+Shishi_outputtype;
+
 typedef enum
 {
   /* 1. AS-REQ PA-ENC-TIMESTAMP padata timestamp, encrypted with the
@@ -525,12 +534,13 @@ struct Shishi_tkts_hint
   char *client;
   char *clientrealm;
   int flags;
+  Shishi_ticketflags tktflags;
+  Shishi_KDCOptions kdcoptions;
   int32_t etype;
   char *passwd;
   time_t starttime;
   time_t endtime;
   time_t renew_till;
-  int renewable;
 };
 typedef struct Shishi_tkts_hint Shishi_tkts_hint;
 
@@ -550,6 +560,7 @@ typedef ASN1_TYPE Shishi_asn1;
 
 /* init.c */
 extern Shishi *shishi (void);
+extern Shishi *shishi_server (void);
 extern void shishi_done (Shishi * handle);
 extern int shishi_init (Shishi ** handle);
 extern int shishi_init_with_paths (Shishi ** handle,
@@ -576,6 +587,8 @@ extern const char *shishi_error (Shishi * handle);
 extern void shishi_error_clear (Shishi * handle);
 extern void shishi_error_set (Shishi * handle, const char *error);
 extern void shishi_error_printf (Shishi * handle, const char *format, ...);
+extern int shishi_outputtype (Shishi * handle);
+extern void shishi_set_outputtype (Shishi * handle, int type);
 extern void shishi_info (Shishi * handle, const char *format, ...);
 extern void shishi_warn (Shishi * handle, const char *format, ...);
 
@@ -667,7 +680,7 @@ extern Shishi_tkt *shishi_tkt2 (Shishi * handle,
 				Shishi_asn1 ticket,
 				Shishi_asn1 enckdcreppart,
 				Shishi_asn1 kdcrep);
-extern int shishi_tkt_pretty_print (Shishi_tkt * tkt, FILE * fh);
+extern void shishi_tkt_pretty_print (Shishi_tkt * tkt, FILE * fh);
 extern int shishi_tkt_realm (Shishi_tkt * tkt, char **realm,
 			     size_t * realmlen);
 extern int shishi_tkt_cnamerealm (Shishi_tkt * tkt, char *cnamerealm,
@@ -684,7 +697,7 @@ extern int shishi_tkt_build (Shishi_tkt * tkt, Shishi_key * key);
 extern int shishi_tkt_lastreq (Shishi_tkt * tkt,
 			       char *lrtime, size_t * lrtimelen, int lrtype);
 extern time_t shishi_tkt_lastreqc (Shishi_tkt * tkt, Shishi_lrtype lrtype);
-extern int shishi_tkt_lastreq_pretty_print (Shishi_tkt * tkt, FILE * fh);
+extern void shishi_tkt_lastreq_pretty_print (Shishi_tkt * tkt, FILE * fh);
 extern int shishi_tkt_authtime (Shishi_tkt * tkt,
 				char *authtime, size_t * authtimelen);
 extern time_t shishi_tkt_authctime (Shishi_tkt * tkt);
@@ -701,7 +714,7 @@ extern int shishi_tkt_keytype (Shishi_tkt * tkt, int32_t * etype);
 extern int shishi_tkt_keytype_p (Shishi_tkt * tkt, int32_t etype);
 extern int shishi_tkt_server (Shishi_tkt * tkt,
 			      char *service, size_t * servicelen);
-extern int shishi_tkt_server_p (Shishi_tkt * tkt, const char *service);
+extern int shishi_tkt_server_p (Shishi_tkt * tkt, const char *server);
 extern int shishi_tkt_server_realm (Shishi_tkt * tkt,
 				    char *serverrealm,
 				    size_t * serverrealmlen);
@@ -759,6 +772,11 @@ extern Shishi_tkt *shishi_tkts_find_for_server (Shishi_tkts * tkts,
 						const char *server);
 extern Shishi_tkt *shishi_tkts_get (Shishi_tkts * tkts,
 				    Shishi_tkts_hint * hint);
+extern Shishi_tkt *shishi_tkts_get_tgt (Shishi_tkts * tkts,
+					Shishi_tkts_hint * hint);
+extern Shishi_tkt *shishi_tkts_get_tgs (Shishi_tkts * tkts,
+					Shishi_tkts_hint * hint,
+					Shishi_tkt *tgt);
 extern Shishi_tkt *shishi_tkts_get_for_clientserver (Shishi_tkts * tkts,
 						     const char *client,
 						     const char *server);
@@ -944,7 +962,8 @@ extern int shishi_tgs_req_build (Shishi_tgs * tgs);
 extern int shishi_tgs_req_process (Shishi_tgs * tgs);
 extern Shishi_asn1 shishi_tgs_rep (Shishi_tgs * tgs);
 extern int shishi_tgs_rep_der (Shishi_tgs * tgs, char **out, size_t * outlen);
-extern int shishi_tgs_rep_build (Shishi_tgs * tgs, Shishi_key * key);
+extern int shishi_tgs_rep_build (Shishi_tgs * tgs, int keyusage,
+				 Shishi_key * key);
 extern int shishi_tgs_rep_process (Shishi_tgs * tgs);
 extern Shishi_asn1 shishi_tgs_krberror (Shishi_tgs * tgs);
 extern int shishi_tgs_krberror_der (Shishi_tgs * tgs, char **out,
@@ -1011,12 +1030,12 @@ extern int shishi_kdcreq_etype (Shishi * handle, Shishi_asn1 kdcreq,
 extern int shishi_kdcreq_set_etype (Shishi * handle, Shishi_asn1 kdcreq,
 				    int32_t * etype, int netype);
 extern int shishi_kdcreq_options (Shishi * handle, Shishi_asn1 kdcreq,
-				  int *flags);
+				  uint32_t *flags);
 extern int shishi_kdcreq_renewable_p (Shishi * handle, Shishi_asn1 kdcreq);
 extern int shishi_kdcreq_options_set (Shishi * handle, Shishi_asn1 kdcreq,
-				      int options);
+				      uint32_t options);
 extern int shishi_kdcreq_options_add (Shishi * handle, Shishi_asn1 kdcreq,
-				      int option);
+				      uint32_t option);
 extern int shishi_kdcreq_clear_padata (Shishi * handle, Shishi_asn1 kdcreq);
 extern int shishi_kdcreq_get_padata (Shishi * handle,
 				     Shishi_asn1 kdcreq,
@@ -1240,6 +1259,30 @@ extern time_t shishi_generalize_ctime (Shishi * handle, const char *t);
 extern int shishi_time (Shishi * handle, Shishi_asn1 node,
 			const char *field, char **time);
 
+/* nettle.c, libgcrypt.c, ... */
+extern int shishi_randomize (Shishi * handle, char *data, size_t datalen);
+extern int shishi_md4 (Shishi * handle, const char *in, size_t inlen,
+		       char *out[16]);
+extern int shishi_md5 (Shishi * handle, const char *in, size_t inlen,
+		       char *out[16]);
+extern int shishi_hmac_md5 (Shishi * handle, const char *key, size_t keylen,
+			    const char *in, size_t inlen, char *outhash[16]);
+extern int shishi_hmac_sha1 (Shishi * handle, const char *key, size_t keylen,
+			     const char *in, size_t inlen, char *outhash[20]);
+extern int shishi_des_cbc_mac (Shishi * handle, const char key[8],
+			       const char iv[8], const char *in, size_t inlen,
+			       char *out[8]);
+extern int shishi_des (Shishi * handle, int decryptp, const char key[8],
+		       const char iv[8], char *ivout[8],
+		       const char *in, size_t inlen, char **out);
+extern int shishi_3des (Shishi * handle, int decryptp, const char key[24],
+			const char iv[8], char *ivout[8],
+			const char *in, size_t inlen, char **out);
+extern int shishi_aes_cts (Shishi * handle, int decryptp,
+			   const char *key, size_t keylen,
+			   const char iv[16], char *ivout[16],
+			   const char *in, size_t inlen, char **out);
+
 /* crypto.c */
 extern int shishi_cipher_supported_p (int type);
 extern const char *shishi_cipher_name (int type);
@@ -1353,7 +1396,6 @@ extern int shishi_verify (Shishi * handle,
 			  int cksumtype,
 			  const char *in, size_t inlen,
 			  const char *cksum, size_t cksumlen);
-extern int shishi_randomize (Shishi * handle, char *data, size_t datalen);
 extern int shishi_dk (Shishi * handle,
 		      Shishi_key * key,
 		      const char *constant, size_t constantlen,
@@ -1364,7 +1406,8 @@ extern int shishi_dr (Shishi * handle,
 		      char *derivedrandom, size_t derivedrandomlen);
 extern int shishi_n_fold (Shishi * handle, const char *in, size_t inlen,
 			  char *out, size_t outlen);
-extern int shishi_pbkdf2_sha1 (const char *P, size_t Plen,
+extern int shishi_pbkdf2_sha1 (Shishi * handle,
+			       const char *P, size_t Plen,
 			       const char *S, size_t Slen,
 			       unsigned int c, unsigned int dkLen, char *DK);
 
@@ -1372,14 +1415,8 @@ extern int shishi_pbkdf2_sha1 (const char *P, size_t Plen,
 extern const char *shishi_check_version (const char *req_version);
 
 /* password.c */
-extern int shishi_read_password (FILE * fh, char *s, int size);
 extern int
-shishi_prompt_password_raw (FILE * in, char *s, int size,
-			    FILE * out, char *format, ...);
-extern int
-shishi_prompt_password (Shishi * handle,
-			FILE * in, char *s, int size,
-			FILE * out, char *format, ...);
+shishi_prompt_password (Shishi * handle, char **s, const char *format, ...);
 
 /* asn1.c */
 extern int shishi_a2d_field (Shishi * handle,
@@ -1404,12 +1441,12 @@ extern int shishi_asn1_read_int32 (Shishi * handle, Shishi_asn1 node,
 extern int shishi_asn1_read_uint32 (Shishi * handle, Shishi_asn1 node,
 				    const char *field, uint32_t * i);
 extern int shishi_asn1_read_bitstring (Shishi * handle, Shishi_asn1 node,
-				       const char *field, int *flags);
+				       const char *field, uint32_t *flags);
 extern int shishi_asn1_read_optional (Shishi * handle,
 				      Shishi_asn1 node, const char *field,
 				      char *data, size_t * datalen);
-extern int shishi_asn1_read_empty_p (Shishi * handle,
-				     Shishi_asn1 node, const char *field);
+extern int shishi_asn1_empty_p (Shishi * handle, Shishi_asn1 node,
+				const char *field);
 extern int shishi_asn1_write (Shishi * handle, Shishi_asn1 node,
 			      const char *field,
 			      const char *data, size_t datalen);
@@ -1420,7 +1457,7 @@ extern int shishi_asn1_write_int32 (Shishi * handle, Shishi_asn1 node,
 extern int shishi_asn1_write_uint32 (Shishi * handle, Shishi_asn1 node,
 				     const char *field, uint32_t n);
 extern int shishi_asn1_write_bitstring (Shishi * handle, Shishi_asn1 node,
-					const char *field, int flags);
+					const char *field, uint32_t flags);
 extern int shishi_asn1_number_of_elements (Shishi * handle,
 					   Shishi_asn1 node,
 					   const char *field, size_t * n);
@@ -1441,9 +1478,11 @@ extern Shishi_asn1 shishi_asn1_krbsafe (Shishi * handle);
 extern Shishi_asn1 shishi_asn1_priv (Shishi * handle);
 extern Shishi_asn1 shishi_asn1_encprivpart (Shishi * handle);
 
+extern Shishi_msgtype shishi_asn1_msgtype (Shishi * handle, Shishi_asn1 node);
+extern Shishi_msgtype shishi_der_msgtype (Shishi * handle,
+					  const char *der, size_t derlen);
+
 extern Shishi_asn1 shishi_der2asn1 (Shishi * handle,
-				    const char *fieldname,
-				    const char *nodename,
 				    const char *der, size_t derlen);
 extern Shishi_asn1 shishi_der2asn1_ticket (Shishi * handle,
 					   const char *der, size_t derlen);
@@ -1879,7 +1918,7 @@ extern int shishi_priv_priv_der_set (Shishi_priv * priv,
 				     char *der, size_t derlen);
 extern Shishi_asn1 shishi_priv_encprivpart (Shishi_priv * priv);
 extern void shishi_priv_encprivpart_set (Shishi_priv * priv,
-					 Shishi_asn1 asn1priv);
+					 Shishi_asn1 asn1encprivpart);
 extern int shishi_priv_encprivpart_der (Shishi_priv * priv, char **out,
 					size_t * outlen);
 extern int shishi_priv_encprivpart_der_set (Shishi_priv * priv,
