@@ -159,11 +159,14 @@ enum
   OPTION_CRYPTO_DEBUG,
   OPTION_CRYPTO_GENERATE_KEY,
   OPTION_CRYPTO,
-  OPTION_VERBOSE_LIBRARY,
   OPTION_LIST,
   OPTION_DESTROY,
   OPTION_RENEW,
   OPTION_RENEWABLE,
+  OPTION_PROXIABLE,
+  OPTION_PROXY,
+  OPTION_FORWARDABLE,
+  OPTION_FORWARDED,
   OPTION_STARTTIME,
   OPTION_ENDTIME,
   OPTION_RENEW_TILL,
@@ -180,7 +183,7 @@ enum
 
 struct arguments
 {
-  int silent, verbose, verbose_library;
+  int silent, verbose;
   char *etypes;
   char *lib_options;
   int command;
@@ -199,6 +202,10 @@ struct arguments
   int forcetgs_p;
   char *servername;
   int renewable;
+  int proxiable;
+  int proxy;
+  int forwardable;
+  int forwarded;
   time_t starttime;
   char *endtime_str;
   time_t endtime;
@@ -488,11 +495,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case 'v':
-      arguments->verbose = 1;
-      break;
-
-    case OPTION_VERBOSE_LIBRARY:
-      arguments->verbose_library = 1;
+      arguments->verbose++;
       break;
 
     case 'o':
@@ -629,6 +632,22 @@ parse_opt (int key, char *arg, struct argp_state *state)
       arguments->endtime_str = strdup (arg);
       break;
 
+    case OPTION_FORWARDABLE:
+      arguments->forwardable = 1;
+      break;
+
+    case OPTION_FORWARDED:
+      arguments->forwarded = 1;
+      break;
+
+    case OPTION_PROXIABLE:
+      arguments->proxiable = 1;
+      break;
+
+    case OPTION_PROXY:
+      arguments->proxy = 1;
+      break;
+
     case OPTION_REALM:
       arguments->crealm = strdup (arg);
       break;
@@ -723,9 +742,23 @@ static struct argp_option options[] = {
    "such as \"2001-02-03 04:05:06 CET\". The default is 8 hours after "
    "the start time.", 0},
 
+  {"forwardable", OPTION_FORWARDABLE, 0, 0,
+   "Get a forwardable ticket, i.e., one that can be used to get forwarded "
+   "tickets.", 0},
+
+  {"forwarded", OPTION_FORWARDED, 0, 0,
+   "Get a forwarded ticket.", 0},
+
   {"list", 'l', 0, 0,
    "List tickets in local cache, subject to --server-name limiting.", 0},
 
+  {"proxiable", OPTION_PROXIABLE, 0, 0,
+   "Get a proxiable ticket, i.e., one that can be used to get proxy "
+   "tickets.", 0},
+
+  {"proxy", OPTION_PROXY, 0, 0,
+   "Get a proxy ticket.", 0},
+
   {"renew", 'R', 0, 0,
    "Renew ticket.  Use --server-name to specify ticket, default is the "
    "most recent renewable ticket granting ticket for the default realm.", 0},
@@ -822,10 +855,8 @@ static struct argp_option options[] = {
   {0, 0, 0, 0, "Other options:", 200},
 
   {"verbose", 'v', 0, 0,
-   "Produce verbose output.", 0},
-
-  {"verbose-library", OPTION_VERBOSE_LIBRARY, 0, 0,
-   "Produce verbose output in the library.", 0},
+   "Produce verbose output.  Use multiple times to increase amount of "
+   "verbose output.", 0},
 
   {"quiet", 'q', 0, 0,
    "Don't produce any output.", 0},
@@ -945,7 +976,7 @@ main (int argc, char *argv[])
     error (1, 0, "Could not read library options: %s\n",
 	   shishi_strerror (rc));
 
-  if (arg.verbose_library)
+  if (arg.verbose > 1)
     {
       rc = shishi_cfg (handle, "verbose");
       if (rc != SHISHI_OK)
@@ -953,6 +984,30 @@ main (int argc, char *argv[])
 	       shishi_strerror (rc));
     }
 
+  if (arg.verbose > 2)
+    {
+      rc = shishi_cfg (handle, "verbose-noice");
+      if (rc != SHISHI_OK)
+	error (1, 0, "Could not make library verbose: %s\n",
+	       shishi_strerror (rc));
+    }
+
+  if (arg.verbose > 3)
+    {
+      rc = shishi_cfg (handle, "verbose-asn1");
+      if (rc != SHISHI_OK)
+	error (1, 0, "Could not make library verbose: %s\n",
+	       shishi_strerror (rc));
+    }
+
+  if (arg.verbose > 4)
+    {
+      rc = shishi_cfg (handle, "verbose-crypto");
+      if (rc != SHISHI_OK)
+	error (1, 0, "Could not make library verbose: %s\n",
+	       shishi_strerror (rc));
+    }
+
   if (!arg.starttime)
     arg.starttime = time (NULL);
 
@@ -1023,12 +1078,15 @@ main (int argc, char *argv[])
 	    i--;
 	    removed++;
 	  }
-	if (removed == 0)
-	  printf ("No tickets removed.\n");
-	else if (removed == 1)
-	  printf ("1 ticket removed.\n");
-	else
-	  printf ("%d tickets removed.\n", removed);
+	if (!arg.silent)
+	  {
+	    if (removed == 0)
+	      printf ("No tickets removed.\n");
+	    else if (removed == 1)
+	      printf ("1 ticket removed.\n");
+	    else
+	      printf ("%d tickets removed.\n", removed);
+	  }
 	rc = SHISHI_OK;
       }
       break;
@@ -1054,21 +1112,15 @@ main (int argc, char *argv[])
 	hint.starttime = arg.starttime;
 	hint.endtime = arg.endtime;
 	hint.renew_till = arg.renew_till;
-	hint.renewable = arg.renewable;
 
 	tkt = shishi_tkts_find (shishi_tkts_default (handle), &hint);
 	if (!tkt)
 	  {
-	    printf ("Could not get ticket for `%s'.\n", hint.server);
+	    fprintf (stderr, "Could not get ticket for `%s'.\n", hint.server);
 	    rc = !SHISHI_OK;
 	  }
 	else
-	  {
-	    rc = shishi_tkt_pretty_print (tkt, stdout);
-	    if (rc != SHISHI_OK)
-	      fprintf (stderr, "Pretty printing ticket failed:\n%s\n%s\n",
-		       shishi_strerror (rc), shishi_error (handle));
-	  }
+	  shishi_tkt_pretty_print (tkt, stdout);
 
 	/* Get ticket using TGT ... */
 	rc = shishi_tgs (handle, &tgs);
@@ -1091,7 +1143,8 @@ main (int argc, char *argv[])
 	  rc = shishi_tgs_rep_process (tgs);
 	if (rc != SHISHI_OK)
 	  {
-	    printf ("TGS exchange failed: %s\n%s\n", shishi_strerror (rc),
+	    fprintf (stderr, "TGS exchange failed: %s\n%s\n",
+		     shishi_strerror (rc),
 		    shishi_error (handle));
 	    if (rc == SHISHI_GOT_KRBERROR)
 	      shishi_krberror_pretty_print (handle, stdout,
@@ -1102,7 +1155,8 @@ main (int argc, char *argv[])
 	tkt = shishi_tgs_tkt (tgs);
 	if (!tkt)
 	  {
-	    printf ("No ticket in TGS-REP?!: %s\n", shishi_error (handle));
+	    fprintf (stderr, "No ticket in TGS-REP?!: %s\n",
+		     shishi_error (handle));
 	    break;
 	  }
 
@@ -1110,7 +1164,7 @@ main (int argc, char *argv[])
 
 	rc = shishi_tkts_add (shishi_tkts_default (handle), tkt);
 	if (rc != SHISHI_OK)
-	  printf ("Could not add ticket: %s", shishi_strerror (rc));
+	  fprintf (stderr, "Could not add ticket: %s", shishi_strerror (rc));
       }
       break;
 
@@ -1125,21 +1179,25 @@ main (int argc, char *argv[])
 	hint.starttime = arg.starttime;
 	hint.endtime = arg.endtime;
 	hint.renew_till = arg.renew_till;
-	hint.renewable = arg.renewable;
+	if (arg.renewable)
+	  hint.tktflags |= SHISHI_TICKETFLAGS_RENEWABLE;
+	if (arg.proxiable)
+	  hint.tktflags |= SHISHI_TICKETFLAGS_PROXIABLE;
+	if (arg.proxy)
+	  hint.tktflags |= SHISHI_TICKETFLAGS_PROXY;
+	if (arg.forwardable)
+	  hint.tktflags |= SHISHI_TICKETFLAGS_FORWARDABLE;
+	if (arg.forwarded)
+	  hint.tktflags |= SHISHI_TICKETFLAGS_FORWARDED;
 
 	tkt = shishi_tkts_get (shishi_tkts_default (handle), &hint);
 	if (!tkt)
 	  {
-	    printf ("Could not get ticket for `%s'.\n", hint.server);
+	    fprintf (stderr, "Could not get ticket for `%s'.\n", hint.server);
 	    rc = !SHISHI_OK;
 	  }
 	else
-	  {
-	    rc = shishi_tkt_pretty_print (tkt, stdout);
-	    if (rc != SHISHI_OK)
-	      fprintf (stderr, "Pretty printing ticket failed:\n%s\n%s\n",
-		       shishi_strerror (rc), shishi_error (handle));
-	  }
+	  shishi_tkt_pretty_print (tkt, stdout);
       }
       break;
     }
